@@ -29,21 +29,48 @@ from .cpp.delaunay_class3 import DelaunayTriangulation3 as DT3
 __all__ = ["sliver_removal", "generate_mesh"]
 
 opts = {
-    "nscreen": 1,
+    "verbose": 1,
     "max_iter": 50,
     "seed": 0,
     "perform_checks": False,
     "pfix": None,
     "axis": 1,
     "min_dh_angle_bound": 10.0,
-    "max_dh_angle_bound": 170.0,
+    "max_dh_angle_bound": 180.0,
     "points": None,
-    "delta_t": 0.20,
+    "delta_t": 0.30,
+    "geps_mult": 0.1,
 }
 
 
-def sliver_removal(points, domain, cell_size, h0, comm=None, **kwargs):
-    r"""Improve an existing 3D mesh by removing degenerate elements
+def silence(func):
+    def wrapper(*args, **kwargs):
+        None
+
+    return wrapper
+
+
+def talk(func):
+    def wrapper(*args, **kwargs):
+        func(*args, **kwargs)
+
+    return wrapper
+
+
+def _select_verbosity(opts):
+    if opts["verbose"] == 0:
+        return silence, silence
+    elif opts["verbose"] == 1:
+        return talk, silence
+    elif opts["verbose"] > 1:
+        return talk, talk
+
+    else:
+        raise ValueError("Unknown verbosity level")
+
+
+def sliver_removal(points, domain, edge_length, comm=None, **kwargs):  # noqa: C901
+    r"""Improve an existing 3D mesh by removing degenerate cells.
     commonly referred to as `slivers`.
 
     :param points:
@@ -51,13 +78,10 @@ def sliver_removal(points, domain, cell_size, h0, comm=None, **kwargs):
     :type points: `np.ndarray`
     :param domain:
         A function that takes a point and returns the signed nearest distance to the domain boundary Ω
-    :type domain: A :class:`geometry.Rectangle/Cube/Circle` object or a function object.
-    :param cell_size:
-        A function that can evalulate a point and return a mesh size.
-    :type cell_size: A :class:`SizeFunction` object or a function object.
-    :param h0:
-        The minimum element size in the domain
-    :type h0: `float`
+    :type domain: A :class:`geometry.Rectangle/Cube/Disk` object or a function object.
+    :param edge_length:
+        A function that can evalulate a point and return an edge length (e.g. length of the triangular edge)
+    :type edge_length: A :class:`SizeFunction` object or a function object.
     :param comm:
         MPI4py communicator
     :type comm: MPI4py communicator object, optional
@@ -66,8 +90,11 @@ def sliver_removal(points, domain, cell_size, h0, comm=None, **kwargs):
         See below
 
     :Keyword Arguments:
-        * *nscreen* (``float``) --
-            Output to the screen `nscreen` timestep. (default==1)
+        * *h0* (``float``) --
+            The minimum  edge length in the domain. REQUIRED IF USING A VARIABLE RESOLUTION EDGE LENGTH.
+        * *verbose* (``int``) --
+            Output to the screen `verbose` (default==1). If `verbose`==1 only start and end messages are
+            written, `verbose`==0, no messages are written besides errors, `verbose` > 1 all messages are written.
         * *max_iter* (``float``) --
             Maximum number of meshing iterations. (default==50)
         * *perform_checks* (`boolean`) --
@@ -80,6 +107,8 @@ def sliver_removal(points, domain, cell_size, h0, comm=None, **kwargs):
             The minimum allowable dihedral angle bound. (default==10 degrees)
         * *max_dh_angle_bound* (`float`) --
             The maximum allowable dihedral angle bound. (default==170 degrees)
+        * *geps_mult* (`float`) --
+            The tolerance used to determine if a vertex is "in" the domain. (default==0.1)
 
     """
     comm = comm or MPI.COMM_WORLD
@@ -91,44 +120,68 @@ def sliver_removal(points, domain, cell_size, h0, comm=None, **kwargs):
     opts.update(kwargs)
     _parse_kwargs(kwargs)
 
+    verbosity1, verbosity2 = _select_verbosity(opts)
+
+    @verbosity1
+    def print_msg1(msg):
+        print(msg, flush=True)
+
+    @verbosity2
+    def print_msg2(msg):
+        print(msg, flush=True)
+
     dim = points.shape[1]
     if dim == 2:
         if comm.rank == 0:
             raise Exception("Mesh improvement currently on works in 3D")
 
-    # unpack domain
     fd, bbox0 = _unpack_domain(domain)
 
-    fh, bbox1 = _unpack_sizing(cell_size)
+    fh, bbox1, hmin = _unpack_sizing(edge_length)
 
-    # take maxmin of boxes
-    bbox = _minmax(bbox0, bbox1)
+    # take minmax of boxes for the case of domain padding
+    if bbox1 is None:
+        bbox = bbox0
+    else:
+        bbox = _minmax(bbox0, bbox1)
 
-    # rebuild the Rectangle or cube if domain padding
-    if bbox0 != bbox1:
+    # rebuild the Rectangle or Cube if domain padding
+    if bbox0 != bbox1 and bbox1 is not None:
         fd = geometry.Cube(bbox).eval
 
     if not isinstance(bbox, tuple):
         raise ValueError("`bbox` must be a tuple")
 
+    # in case of scalar sizing function
+    if hmin is not None:
+        h0 = hmin
+    else:
+        h0 = opts["h0"]
+
+    # check h0
+    if h0 < 0:
+        raise ValueError("`h0` must be > 0")
+
+    deps = np.sqrt(np.finfo(np.double).eps) * h0
+
     if opts["max_iter"] < 0:
         raise ValueError("`max_iter` must be > 0")
     max_iter = opts["max_iter"]
-    print(
+
+    print_msg1(
         "Will attempt " + str(max_iter) + " iterations to bound the dihedral angles..."
     )
 
-    geps = 1e-1 * h0
-    deps = np.sqrt(np.finfo(np.double).eps) * h0
+    geps = opts["geps_mult"] * h0
     min_dh_bound = opts["min_dh_angle_bound"] * math.pi / 180
     max_dh_bound = opts["max_dh_angle_bound"] * math.pi / 180
 
-    print(
+    print_msg1(
         "Enforcing a min. dihedral bound of: "
         + str(min_dh_bound * 180 / math.pi)
         + " degrees..."
     )
-    print(
+    print_msg1(
         "Enforcing a max. dihedral bound of: "
         + str(max_dh_bound * 180 / math.pi)
         + " degrees..."
@@ -140,24 +193,23 @@ def sliver_removal(points, domain, cell_size, h0, comm=None, **kwargs):
 
     N = len(p)
 
-    print(
-        "Commencing sliver removal with %d vertices on rank %d." % (N, comm.rank),
-        flush=True,
+    print_msg1(
+        "Commencing sliver removal with %d vertices on rank %d." % (N, comm.rank)
     )
 
     count = 0
     pold = None
-    nscreen = opts["nscreen"]
+    push = 0.10
 
+    dt = DT()
+    dt.insert(p.flatten().tolist())
     while True:
 
         start = time.time()
 
+        num_move = 0
         # Using CGAL's incremental Delaunay triangulation capabilities.
-        if count == 0:
-            dt = DT()
-            dt.insert(p.flatten().tolist())
-        else:
+        if count > 0:
             to_move = np.where(_dist(p, pold) > 0)[0]
             dt.move(to_move.flatten().tolist(), p[to_move].flatten().tolist())
 
@@ -169,87 +221,76 @@ def sliver_removal(points, domain, cell_size, h0, comm=None, **kwargs):
         # Remove points outside the domain
         t = _remove_triangles_outside(p, t, fd, geps)
 
-        # Sliver removal
-        if count != (max_iter - 1):
-            num_move = 0
-            # calculate dihedral angles in mesh
-            dh_angles = geometry.calc_dihedral_angles(p, t)
-            out_of_bounds = np.argwhere(
-                (dh_angles[:, 0] < min_dh_bound) | (dh_angles[:, 0] > max_dh_bound)
-            )
-            ele_nums = np.floor(out_of_bounds / 6).astype("int")
-            ele_nums, ix = np.unique(ele_nums, return_index=True)
-
-            if count % nscreen == 0:
-                print(
-                    "On rank: "
-                    + str(comm.rank)
-                    + " There are "
-                    + str(len(ele_nums))
-                    + " slivers...",
-                    flush=True,
-                )
-
-            move = t[ele_nums, 0]
-            num_move = move.size
-            if num_move == 0:
-                print(
-                    "Termination reached in "
-                    + str(count)
-                    + " iterations...no slivers detected!",
-                    flush=True,
-                )
-                p, t, _ = geometry.fix_mesh(p, t, dim=dim, delete_unused=True)
-                return p, t
-
-            p0, p1, p2, p3 = (
-                p[t[ele_nums, 0], :],
-                p[t[ele_nums, 1], :],
-                p[t[ele_nums, 2], :],
-                p[t[ele_nums, 3], :],
-            )
-
-            # perturb vector is based on INCREASING circumsphere's radius
-            perturb = geometry.calc_circumsphere_grad(p0, p1, p2, p3)
-            perturb[np.isinf(perturb)] = 1.0
-
-            # normalize perturbation vector
-            perturb_norm = np.sum(np.abs(perturb) ** 2, axis=-1) ** (1.0 / 2)
-            perturb /= perturb_norm[:, None]
-
-            # perturb % of local mesh size
-            p[move] += 0.10 * h0 * perturb
-
-        # Bring outside points back to the boundary
-        p = _project_points_back(p, fd, deps)
+        ele_nums, ix = _calc_dihedral_angles(p, t, min_dh_bound, max_dh_bound)
 
         # Number of iterations reached, stop.
         if count == (max_iter - 1):
+            print_msg1(
+                "FAILURE: Termination...maximum number of iterations reached.",
+            )
             p, t = _termination(p, t, opts, comm, sliver=True)
             break
 
+        print_msg1(
+            "On rank: "
+            + str(comm.rank)
+            + " There are "
+            + str(len(ele_nums))
+            + " slivers...",
+        )
+
+        move = t[ele_nums, 0]
+
+        num_move = move.size
+
+        if num_move == 0:
+            print_msg1(
+                "Termination reached in "
+                + str(count)
+                + " iterations...no slivers detected!",
+            )
+            p, t, _ = geometry.fix_mesh(p, t, dim=dim, delete_unused=True)
+            return p, t
+
+        p0, p1, p2, p3 = (
+            p[t[ele_nums, 0], :],
+            p[t[ele_nums, 1], :],
+            p[t[ele_nums, 2], :],
+            p[t[ele_nums, 3], :],
+        )
+
+        # perturb vector is based on INCREASING the sliver's circumsphere radius
+        perturb = geometry.calc_circumsphere_grad(p0, p1, p2, p3)
+        perturb[np.isinf(perturb)] = 1.0
+
+        # normalize the perturbation vector
+        perturb_norm = np.sum(np.abs(perturb) ** 2, axis=-1) ** (1.0 / 2)
+        perturb /= perturb_norm[:, None]
+
+        # perturb push % of minimum mesh size
+        p[move] += push * h0 * perturb
+
+        # bring outside points back to the boundary
+        p = _project_points_back_newton(p, fd, deps)
+
         count += 1
 
         end = time.time()
-        if comm.rank == 0 and count % nscreen == 0:
-            print("     Elapsed wall-clock time %f : " % (end - start), flush=True)
+        if comm.rank == 0:
+            print_msg2("     Elapsed wall-clock time %f : " % (end - start))
 
     return p, t
 
 
-# @profile
-def generate_mesh(domain, cell_size, h0, comm=None, **kwargs):
-    r"""Generate a 2D/3D triangulation using callbacks to a sizing function `cell_size` and signed distance function :class:`domain`
+def generate_mesh(domain, edge_length, comm=None, **kwargs):  # noqa: C901
+    r"""Generate a 2D/3D mesh using callbacks to a sizing function `edge_length` and signed distance function `domain`
 
     :param domain:
         A function that takes a point and returns the signed nearest distance to the domain boundary Ω.
-    :type domain: A :class:`geometry.Rectangle/Cube/Circle` object or a function object.
-    :param cell_size:
-        Either a :class:`size_function` object or a function that can evalulate a point and return a mesh size.
-    :type cell_size: A :class:`cell_size` object or a function object.
-    :param h0:
-        The minimum element size in the domain.
-    :type h0: `float`
+    :type domain: A :class:`geometry.Rectangle/Cube/Disk` object or a function object.
+    :param edge_length:
+        Edge lengths in the domain (e.g., triangular edge lengths assuming all triangles are equilateral).
+    :type edge_length: A :class:`SizeFunction` object, a function object, or a scalar value.
     :param comm:
         MPI4py communicator
     :type comm: MPI4py communicator object, optional
@@ -258,12 +299,15 @@ def generate_mesh(domain, cell_size, h0, comm=None, **kwargs):
         See below
 
     :Keyword Arguments:
+        * *h0* (``float``) --
+            The minimum edge length in the domain. REQUIRED IF USING A VARIABLE RESOLUTION EDGE LENGTH
         * *bbox* (``tuple``) --
-            Bounding box containing domain extents. REQUIRED IF NOT USING :class:`cell_size`
-        * *nscreen* (``float``) --
-            Output to the screen `nscreen` timestep. (default==1)
+            Bounding box containing domain extents. REQUIRED IF NOT USING :class:`edge_length`
+        * *verbose* (``int``) --
+            Output to the screen `verbose` (default==1). If `verbose`==1 only start and end messages are
+            written, `verbose`==0, no messages are written besides errors, `verbose` > 1 all messages are written.
         * *max_iter* (``float``) --
-            Maximum number of meshing iterations. (default==50)
+           Maximum number of meshing iterations. (default==50)
         * *seed* (``float`` or ``int``) --
             Psuedo-random seed to initialize meshing points. (default==0)
         * *perform_checks* (`boolean`) --
@@ -271,9 +315,12 @@ def generate_mesh(domain, cell_size, h0, comm=None, **kwargs):
         * *pfix* (`array-like`) --
             An array of points to constrain in the mesh. (default==None)
         * *axis* (`int`) --
-            The axis to decompose the mesh (1,2, or 3). (default==1)
+            The axis to decompose the mesh (0,1, or 2). (default==1)
         * *delta_t* (`float`) --
-            Psuedo-timestep to control movement of points (default=0.10)
+            Psuedo-timestep to control movement of points (default=0.30)
+        * *geps_mult* (`float`) --
+            The tolerance used to determine if a vertex is "in" the domain. (default==0.1*h0)
+
 
     :return: points: vertex coordinates of mesh
     :rtype: points: (numpy.ndarray[`float` x dim])
@@ -287,13 +334,27 @@ def generate_mesh(domain, cell_size, h0, comm=None, **kwargs):
     opts.update(kwargs)
     _parse_kwargs(kwargs)
 
+    # verbosity decorators
+    verbosity1, verbosity2 = _select_verbosity(opts)
+
+    @verbosity1
+    def print_msg1(msg):
+        print(msg, flush=True)
+
+    @verbosity2
+    def print_msg2(msg):
+        print(msg, flush=True)
+
     # unpack domain
     fd, bbox0 = _unpack_domain(domain)
 
-    fh, bbox1 = _unpack_sizing(cell_size)
+    fh, bbox1, hmin = _unpack_sizing(edge_length)
 
-    # take maxmin of boxes
-    bbox = _minmax(bbox0, bbox1)
+    # take maxmin of boxes for the case of domain padding
+    if bbox1 is None:
+        bbox = bbox0
+    else:
+        bbox = _minmax(bbox0, bbox1)
 
     if not isinstance(bbox, tuple):
         raise ValueError("`bbox` must be a tuple")
@@ -302,7 +363,7 @@ def generate_mesh(domain, cell_size, h0, comm=None, **kwargs):
     dim = int(len(bbox) / 2)
 
     # rebuild the Rectangle or Cube if domain padding
-    if bbox0 != bbox1:
+    if bbox0 != bbox1 and bbox1 is not None:
         if dim == 2:
             tmp = geometry.Rectangle(bbox)
         elif dim == 3:
@@ -312,6 +373,11 @@ def generate_mesh(domain, cell_size, h0, comm=None, **kwargs):
 
     bbox = np.array(bbox).reshape(-1, 2)
 
+    if hmin is not None:
+        h0 = hmin
+    else:
+        h0 = opts["h0"]
+
     # check h0
     if h0 < 0:
         raise ValueError("`h0` must be > 0")
@@ -320,10 +386,10 @@ def generate_mesh(domain, cell_size, h0, comm=None, **kwargs):
         raise ValueError("`max_iter` must be > 0")
     max_iter = opts["max_iter"]
 
-    """These parameters originate from the original DistMesh"""
+    # these parameters originate from the original DistMesh
     L0mult = 1 + 0.4 / 2 ** (dim - 1)
     delta_t = opts["delta_t"]
-    geps = 1e-1 * h0
+    geps = opts["geps_mult"] * h0
     deps = np.sqrt(np.finfo(np.double).eps) * h0
 
     DT = _select_cgal_dim(dim)
@@ -337,12 +403,11 @@ def generate_mesh(domain, cell_size, h0, comm=None, **kwargs):
     assert N > 0, "No vertices to mesh with!"
 
     count = 0
-    print(
+
+    print_msg1(
         "Commencing mesh generation with %d vertices on rank %d." % (N, comm.rank),
-        flush=True,
     )
 
-    nscreen = opts["nscreen"]
     while True:
 
         start = time.time()
@@ -354,6 +419,12 @@ def generate_mesh(domain, cell_size, h0, comm=None, **kwargs):
         # Get the current topology of the triangulation
         p, t = _get_topology(dt)
 
+        # Find where pfix went
+        ifix = []
+        if nfix > 0:
+            for fix in pfix:
+                ifix.append(_closest_node(fix, p))
+
         # Add ghost points to perform Delaunay in parallel.
         if comm.size > 1:
             p, t, inv, recv_ix = _add_ghost_vertices(p, t, dt, extents, comm)
@@ -363,38 +434,58 @@ def generate_mesh(domain, cell_size, h0, comm=None, **kwargs):
 
         # Number of iterations reached, stop.
         if count == (max_iter - 1):
+            if comm.rank == 0:
+                print_msg1(
+                    "Termination reached...maximum number of iterations reached.",
+                )
             p, t = _termination(p, t, opts, comm)
+            if comm.rank == 0:
+                p = _improve_level_set_newton(p, t, fd, deps, deps * 1000)
             break
 
         # Compute the forces on the edges
         Ftot = _compute_forces(p, t, fh, h0, L0mult)
 
-        Ftot[:nfix] = 0  # Force = 0 at fixed points
+        Ftot[ifix] = 0  # Force = 0 at fixed points
 
         # Update positions
         p += delta_t * Ftot
 
         # Bring outside points back to the boundary
-        p = _project_points_back(p, fd, deps)
+        p = _project_points_back_newton(p, fd, deps)
 
         if comm.size > 1:
             # If continuing on, delete ghost points
             p = np.delete(p, inv[-recv_ix::], axis=0)
 
         # Show the user some progress so they know something is happening
-        if count % nscreen == 0 and comm.rank == 0:
+        if comm.rank == 0:
             maxdp = delta_t * np.sqrt((Ftot ** 2).sum(1)).max()
-            _display_progress(p, t, count, nscreen, maxdp, comm)
+            print_msg2(
+                "Iteration #%d, max movement is %f, there are %d vertices and %d cells"
+                % (count + 1, maxdp, len(p), len(t)),
+            )
 
         count += 1
 
         end = time.time()
-        if comm.rank == 0 and count % nscreen == 0:
-            print("     Elapsed wall-clock time %f : " % (end - start), flush=True)
+        if comm.rank == 0:
+            print_msg2("     Elapsed wall-clock time %f : " % (end - start))
 
     return p, t
 
 
+def _calc_dihedral_angles(p, t, min_dh_bound, max_dh_bound):
+    """calculate the minimum dihedral angle in mesh"""
+    dh_angles = geometry.calc_dihedral_angles(p, t)
+    out_of_bounds = np.argwhere(
+        (dh_angles[:, 0] < min_dh_bound) | (dh_angles[:, 0] > max_dh_bound)
+    )
+    ele_nums = np.floor(out_of_bounds / 6).astype("int")
+    ele_nums, ix = np.unique(ele_nums, return_index=True)
+    return ele_nums, ix
+
+
 def _minmax(bbox0, bbox1):
     d = []
     for i, (a, b) in enumerate(zip(bbox0, bbox1)):
@@ -407,18 +498,38 @@ def _minmax(bbox0, bbox1):
     return tuple(d)
 
 
-def _unpack_sizing(cell_size):
-    if isinstance(cell_size, sizing.SizeFunction):
-        bbox = cell_size.bbox
-        fh = cell_size.eval
-    elif callable(cell_size):
-        bbox = opts["bbox"]
-        fh = cell_size
+def _check_bbox(bbox):
+    if bbox is not None:
+        for b in bbox:
+            if isinstance(b, int):
+                raise ValueError("bbox must contain all floats")
+
+
+def _unpack_sizing(edge_length):
+    bbox = None
+    hmin = None
+    if isinstance(edge_length, sizing.SizeFunction):
+        bbox = edge_length.bbox
+        fh = edge_length.eval
+    elif callable(edge_length):
+        fh = edge_length
+    elif np.isscalar(edge_length):
+
+        def func(x):
+            if type(x) == tuple:
+                h = np.zeros_like(x[0]) + edge_length
+            else:
+                h = np.array([edge_length] * len(x))
+            return h
+
+        fh = func
+        hmin = edge_length
     else:
         raise ValueError(
-            "`cell_size` must either be a function or a `cell_size` object"
+            "`edge_length` must either be a function, a `edge_length` object, or a scalar"
         )
-    return fh, bbox
+    _check_bbox(bbox)
+    return fh, bbox, hmin
 
 
 def _unpack_domain(domain):
@@ -428,7 +539,7 @@ def _unpack_domain(domain):
     elif isinstance(domain, geometry.Cube):
         bbox = (domain.x1, domain.x2, domain.y1, domain.y2, domain.z1, domain.z2)
         fd = domain.eval
-    elif isinstance(domain, geometry.Circle):
+    elif isinstance(domain, geometry.Disk):
         bbox = (domain.x1, domain.x2, domain.y1, domain.y2)
         fd = domain.eval
     elif callable(domain):
@@ -437,13 +548,14 @@ def _unpack_domain(domain):
         fd = domain
     else:
         raise ValueError("`domain` must be a function or a :class:`geometry` object")
+    _check_bbox(bbox)
     return fd, bbox
 
 
 def _parse_kwargs(kwargs):
     for key in kwargs:
         if key in {
-            "nscreen",
+            "verbose",
             "max_iter",
             "seed",
             "perform_checks",
@@ -451,11 +563,13 @@ def _parse_kwargs(kwargs):
             "axis",
             "points",
             "domain",
-            "cell_size",
+            "edge_length",
             "bbox",
             "min_dh_angle_bound",
             "max_dh_angle_bound",
             "delta_t",
+            "h0",
+            "geps_mult",
         }:
             pass
         else:
@@ -464,20 +578,9 @@ def _parse_kwargs(kwargs):
             )
 
 
-def _display_progress(p, t, count, nscreen, maxdp, comm):
-    """print progress"""
-    print(
-        "Iteration #%d, max movement is %f, there are %d vertices and %d cells"
-        % (count + 1, maxdp, len(p), len(t)),
-        flush=True,
-    )
-
-
 def _termination(p, t, opts, comm, sliver=False):
     """Shut it down when reacing `max_iter`"""
     dim = p.shape[1]
-    if comm.rank == 0:
-        print("Termination reached...maximum number of iterations reached.", flush=True)
     if comm.size > 1 and sliver is False:
         # gather onto rank 0
         p, t = migration.aggregate(p, t, comm, comm.size, comm.rank, dim=dim)
@@ -551,8 +654,32 @@ def _remove_triangles_outside(p, t, fd, geps):
     return t[fd(pmid) < -geps]  # Keep interior triangles
 
 
-def _project_points_back(p, fd, deps):
-    """Project points outsidt the domain back within"""
+def _improve_level_set_newton(p, t, fd, deps, tol):
+    """Reduce level set error by using Newton's minimization method"""
+    dim = p.shape[1]
+    max, abs = np.amax, np.abs
+    bid = geometry.get_boundary_vertices(t, dim)
+    it = 0
+    d = fd(p[bid])
+    while max(abs(d)) > tol and it < 10:
+
+        def _deps_vec(i):
+            a = [0] * dim
+            a[i] = deps
+            return a
+
+        dgrads = [(fd(p[bid] + _deps_vec(i)) - d) / deps for i in range(dim)]
+        dgrad2 = sum(dgrad ** 2 for dgrad in dgrads)
+        dgrad2 = np.where(dgrad2 < deps, deps, dgrad2)
+        p[bid] -= (d * np.vstack(dgrads) / dgrad2).T  # Project
+        it += 1
+    return p
+
+
+def _project_points_back_newton(p, fd, deps):
+    """Project points outside the domain back with one iteration of Newton minimization method
+    finding the root of f(p)
+    """
     dim = p.shape[1]
 
     d = fd(p)
@@ -601,13 +728,7 @@ def _generate_initial_points(h0, geps, dim, bbox, fh, fd, pfix, comm, opts):
         p = mutils.make_init_points(bbox, comm.rank, comm.size, opts["axis"], h0, dim)
     else:
         # Create initial distribution in bounding box (equilateral triangles)
-        if dim == 2:
-            p = mutils.create_staggered_grid_2d(h0, bbox)
-        elif dim == 3:
-            p = np.mgrid[tuple(slice(min, max + h0, h0) for min, max in bbox)].astype(
-                float
-            )
-            p = p.reshape(dim, -1).T
+        p = mutils.create_staggered_grid(h0, dim, bbox)
 
     # Remove points outside the region, apply the rejection method
     p = p[fd(p) < geps]  # Keep only d<0 points
@@ -646,8 +767,8 @@ def _form_extents(p, h0, comm):
     if comm.size > 1:
         # min x min y min z max x max y max z
         extent = [*np.amin(p, axis=0), *np.amax(p, axis=0)]
-        extent[_axis] -= h0
-        extent[_axis + dim] += h0
+        extent[_axis] -= 5 * h0
+        extent[_axis + dim] += 5 * h0
         return [comm.bcast(extent, r) for r in range(comm.size)]
     else:
         return []
@@ -690,3 +811,10 @@ def _get_topology(dt):
     p = dt.get_finite_vertices()
     t = dt.get_finite_cells()
     return p, t
+
+
+def _closest_node(node, nodes):
+    nodes = np.asarray(nodes)
+    deltas = nodes - node
+    dist_2 = np.einsum("ij,ij->i", deltas, deltas)
+    return np.argmin(dist_2)
