@@ -6,7 +6,7 @@ from .cpp.fast_geometry import (
     calc_volume_grad,
     unique_edges,
 )
-from .signed_distance_functions import Rectangle, Cube, Circle, drectangle, dblock
+from .signed_distance_functions import Rectangle, Cube, Disk, drectangle, dblock
 from .utils import (
     calc_re_ratios,
     delete_boundary_entities,
@@ -28,6 +28,7 @@ from .utils import (
     unique_rows,
     vertex_to_entities,
     vertex_in_entity3,
+    corners,
 )
 
 __all__ = [
@@ -36,9 +37,10 @@ __all__ = [
     "calc_circumsphere_grad",
     "calc_3x3determinant",
     "calc_4x4determinant",
+    "corners",
     "Rectangle",
     "Cube",
-    "Circle",
+    "Disk",
     "drectangle",
     "dblock",
     "calc_dihedral_angles",
