@@ -1,18 +1,18 @@
 import numpy as np
 
 
-class Circle:
-    def __init__(self, xc, yc, r):
-        self.xc = xc
-        self.yc = yc
+class Disk:
+    def __init__(self, x0, r):
+        self.xc = x0[0]
+        self.yc = x0[1]
         self.r = r
-        self.x1 = xc - r
-        self.x2 = xc + r
-        self.y1 = yc - r
-        self.y2 = yc + r
+        self.x1 = x0[0] - r
+        self.x2 = x0[0] + r
+        self.y1 = x0[1] - r
+        self.y2 = x0[1] + r
 
     def eval(self, x):
-        return _dcircle(x, self.xc, self.yc, self.r)
+        return _ddisk(x, self.xc, self.yc, self.r)
 
 
 class Rectangle:
@@ -39,8 +39,8 @@ class Cube:
         return dblock(x, self.x1, self.x2, self.y1, self.y2, self.z1, self.z2)
 
 
-def _dcircle(p, xc, yc, r):
-    """Signed distance to circle centered at xc, yc with radius r."""
+def _ddisk(p, xc, yc, r):
+    """Signed distance to disk centered at xc, yc with radius r."""
     return np.sqrt(((p - np.array([xc, yc])) ** 2).sum(-1)) - r
 
 
@@ -53,6 +53,40 @@ def drectangle(p, x1, x2, y1, y2):
     return -min(min(min(-y1 + p[:, 1], y2 - p[:, 1]), -x1 + p[:, 0]), x2 - p[:, 0])
 
 
+def dblock0(p, x1, x2, y1, y2, z1, z2):
+    # adapted from:
+    # https://github.com/nschloe/dmsh/blob/3305c417d373d509c78491b24e77409411aa18c2/dmsh/geometry/rectangle.py#L31
+    # outside dist
+    # https://gamedev.stackexchange.com/a/44496
+    w = x2 - x1
+    h = y2 - y1
+    d = z2 - z1
+    cx = (x1 + x2) / 2
+    cy = (y1 + y2) / 2
+    cz = (z1 + z2) / 2
+    dx = np.abs(p[:, 0] - cx) - w / 2
+    dy = np.abs(p[:, 1] - cy) - h / 2
+    dz = np.abs(p[:, 2] - cz) - d / 2
+    is_inside = (dx <= 0) & (dy <= 0) & (dz <= 0)
+    dx[dx < 0.0] = 0.0
+    dy[dy < 0.0] = 0.0
+    dz[dz < 0.0] = 0.0
+    dist = np.sqrt(dx ** 2 + dy ** 2 + dz ** 2)
+    # inside dist
+    a = np.array(
+        [
+            p[is_inside, 0] - x1,
+            x2 - p[is_inside, 0],
+            p[is_inside, 1] - y1,
+            y2 - p[is_inside, 1],
+            p[is_inside, 2] - z1,
+            z2 - p[is_inside, 2],
+        ]
+    )
+    dist[is_inside] = -np.min(a, axis=0)
+    return dist
+
+
 def dblock(p, x1, x2, y1, y2, z1, z2):
     min = np.minimum
     return -min(
