@@ -50,6 +50,8 @@ opts = {
     "nx": None,
     "ny": None,
     "byte_order": "byte_order",
+    "axes_order": (0, 1, 2),
+    "axes_order_sort": "F",
 }
 
 
@@ -67,9 +69,9 @@ def get_sizing_function_from_segy(filename, bbox, comm=None, **kwargs):
 
     :Keyword Arguments:
         * *hmin* (``float``) --
-            Minimum element size in the domain (default==150 m)
+            Minimum edge length in the domain (default==150 m)
         * *hmax* (``float``) --
-            Maximum element size in the domain (default==10,000 m)
+            Maximum edge length in the domain (default==10,000 m)
         * *wl* (``int``) --
             Number of vertices per wavelength for a given 𝑓𝑚𝑎𝑥 (default==0 vertices)
         * *freq* (``float``) --
@@ -98,6 +100,10 @@ def get_sizing_function_from_segy(filename, bbox, comm=None, **kwargs):
              REQUIRED FOR BINARY VELOCITY MODEL. The number of grid points in the x-direction in the velocity model.
         * *byte_order* (``string``) --
              REQUIRED FOR BINARY VELOCITY MODEL. The order of bytes in a 3D sesimic velocity model (`little` or `big`).
+        * *axes_order* (``tuple``) --
+             The order of the axes (assumes z,x,y is (0,1,2))
+        * *axes_order_sort* (``string``) --
+             The sort style of the data (either "F" for FORTRAN-style or "C" for C-style)
 
     :return: a :class:`SizeFunction` object with a `obj.bbox` field and an `obj.eval` method.
     :rtype: a :class:`SizeFunction` object
@@ -115,6 +121,8 @@ def get_sizing_function_from_segy(filename, bbox, comm=None, **kwargs):
             nx=opts["nx"],
             ny=opts["ny"],
             byte_order=opts["byte_order"],
+            axes_order=opts["axes_order"],
+            axes_order_sort=opts["axes_order_sort"],
         )
 
         if opts["units"] == "km-s":
@@ -148,6 +156,8 @@ def get_sizing_function_from_segy(filename, bbox, comm=None, **kwargs):
                 "nx",
                 "ny",
                 "byte_order",
+                "axes_order",
+                "axes_order_sort",
             }:
                 pass
             else:
@@ -160,10 +170,10 @@ def get_sizing_function_from_segy(filename, bbox, comm=None, **kwargs):
                 _gradient_sizing(vp, opts["grad"]),
             )
 
-        print("Enforcing minimum element size of " + str(opts["hmin"]))
+        print("Enforcing minimum edge length of " + str(opts["hmin"]))
         cell_size[cell_size < opts["hmin"]] = opts["hmin"]
 
-        print("Enforcing maximum element size of " + str(opts["hmax"]))
+        print("Enforcing maximum edge length of " + str(opts["hmax"]))
         cell_size[cell_size > opts["hmax"]] = opts["hmax"]
 
         cell_size = _enforce_courant_sizing(
@@ -217,7 +227,10 @@ def write_velocity_model(filename, ofname=None, comm=None, **kwargs):
              Width of the domain pad in meters.
         * *pad_style* (``string``) --
              Type of padding.
-
+        * *axes_order* (``tuple``) --
+             The order of the axes (assumes z,x,y is (0,1,2))
+        * *axes_order_sort* (``string``) --
+             The sort style of the data (either "F" for FORTRAN-style or "C" for C-style)
 
     """
     comm = comm or MPI.COMM_WORLD
@@ -234,6 +247,8 @@ def write_velocity_model(filename, ofname=None, comm=None, **kwargs):
             nx=opts["nx"],
             ny=opts["ny"],
             byte_order=opts["byte_order"],
+            axes_order=opts["axes_order"],
+            axes_order_sort=opts["axes_order_sort"],
         )
 
         if opts["domain_pad"] > 0.0:
@@ -490,20 +505,30 @@ def _pad_it(array, padding, style, extra):
     return array
 
 
-def _read_velocity_model(filename, nz=None, nx=None, ny=None, byte_order=None):
+def _read_velocity_model(
+    filename,
+    nz=None,
+    nx=None,
+    ny=None,
+    byte_order=None,
+    axes_order=None,
+    axes_order_sort=None,
+):
     """Read a velocity model"""
     if filename.endswith(".segy"):
         return _read_segy(filename)
     else:
-        return _read_bin(filename, nz, nx, ny, byte_order)
+        return _read_bin(filename, nz, nx, ny, byte_order, axes_order, axes_order_sort)
 
 
-def _read_bin(filename, nz, nx, ny, byte_order):
+def _read_bin(filename, nz, nx, ny, byte_order, axes_order, axes_order_sort):
     """Read a velocity model from a binary"""
     if (nz is None) or (nx is None) or (ny is None):
         raise ValueError(
             "Please specify the number of grid points in each dimension (e.g., `nz`, `nx`, `ny`)..."
         )
+    axes = [nz, nx, ny]
+    axes = [axes[o] for o in axes_order]
     with open(filename, "r") as file:
         print("Reading binary file: " + filename)
         if byte_order == "big":
@@ -512,8 +537,8 @@ def _read_bin(filename, nz, nx, ny, byte_order):
             vp = np.fromfile(file, dtype=np.dtype("float32").newbyteorder("<"))
         else:
             raise ValueError("Please specify byte_order as either: little or big.")
-        vp = vp.reshape(nx, ny, nz, order="F")
-        return np.flipud(vp.transpose((2, 0, 1))), nz, nx, ny  # z, x and then y
+        vp = vp.reshape(*axes, order=axes_order_sort)
+        return np.flipud(vp.transpose((*axes_order))), nz, nx, ny  # z, x and then y
 
 
 def _read_segy(filename):
@@ -524,7 +549,7 @@ def _read_segy(filename):
         for index, trace in enumerate(f.trace):
             vp[:, index] = trace
         if np.amin(vp) < 1000.0:
-            raise Warning(
+            warnings.warn(
                 "Velocity appear to be in km/s. Maybe pass `units` km-s key pair?"
             )
         return np.flipud(vp), nz, nx, 0
