@@ -16,7 +16,7 @@ from SeismicMesh import (
 def test_2dmesher():
 
     fname = os.path.join(os.path.dirname(__file__), "testing.segy")
-    bbox = (-10e3, 0.0, 0.0, 10e3)
+    bbox = (-10000.0, 0.0, 0.0, 10000.0)
     wl = 5
     freq = 5.0
     hmin = 100
@@ -40,7 +40,7 @@ def test_2dmesher():
     points, cells = generate_mesh(
         rectangle,
         ef,
-        hmin,
+        h0=hmin,
         perform_checks=True,
     )
     # should have: 7690 vertices and 15045 cells
