@@ -21,7 +21,7 @@ def test_3dmesher():
     freq = 2
     hmin = 50
     grade = 0.005
-    bbox = (-2e3, 0.0, 0.0, 1e3, 0, 1e3)
+    bbox = (-2e3, 0.0, 0.0, 1e3, 0.0, 1e3)
     cube = Cube(bbox)
     ef = get_sizing_function_from_segy(
         fname,
@@ -36,6 +36,7 @@ def test_3dmesher():
         ny=10,
         byte_order="little",
         domain_pad=0.0,
+        axes_order=(2, 0, 1),
     )
 
     write_velocity_model(
@@ -44,13 +45,13 @@ def test_3dmesher():
 
     points, cells = generate_mesh(
         domain=cube,
-        cell_size=ef,
+        edge_length=ef,
         h0=hmin,
         max_iter=25,
         perform_checks=False,
     )
 
-    points, cells = sliver_removal(points=points, cell_size=ef, domain=cube, h0=hmin)
+    points, cells = sliver_removal(points=points, edge_length=ef, domain=cube, h0=hmin)
     print(len(points), len(cells))
     allclose([len(points), len(cells)], [16459, 89240], atol=100)
 
